from setuptools import setup, find_packages

setup(
    name='MoyeMoye',  # The name of your package
    version='0.2',  # The version of your package
    packages=['MoyeMoye'],  # Automatically find the packages in your directory
    description='Your package description',  # A brief description of your package
    author='DZANUM',  # Your name
    author_email='random.email@example.com',  # Your email
      
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
